﻿// Anyone is free to copy, modify, use, compile, or distribute this software,
// either in source code form or as a compiled binary, for any non-commercial
// purpose.  For commercial enquiries please contact contact@gimpysoftware.com
//
// If modified, please retain this copyright header, and consider telling us
// about your changes.  We're always glad to see how people use our code!
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
// Gimpy Software does not accept any liability for damage caused by executing
// or modifying this code.
//
// Copyright (c) 2013, Gimpy Software.
//
using System.Diagnostics;

namespace Speculator
{
    public class ClockSync
    {
        private readonly Stopwatch m_realTime;
        private long m_emulatedTicks;
        private readonly double m_emulatedTicksPerSecond;

        public ClockSync(double emulatedCpuMHz)
        {
            m_realTime = Stopwatch.StartNew();
            m_emulatedTicksPerSecond = emulatedCpuMHz;
        }

        public void SyncWithRealTime()
        {
            double emulatedUptimeSecs = m_emulatedTicks / m_emulatedTicksPerSecond;
            var targetRealUptimeTicks = Stopwatch.Frequency * emulatedUptimeSecs;

            while (m_realTime.ElapsedTicks < targetRealUptimeTicks)
            {
                // Absolutely nothing.
            }
        }

        public void IncrementEmulationTicks(long ticks)
        {
            m_emulatedTicks += ticks;
        }
    }
}
