﻿// Anyone is free to copy, modify, use, compile, or distribute this software,
// either in source code form or as a compiled binary, for any non-commercial
// purpose.  For commercial enquiries please contact contact@gimpysoftware.com
//
// If modified, please retain this copyright header, and consider telling us
// about your changes.  We're always glad to see how people use our code!
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
// Gimpy Software does not accept any liability for damage caused by executing
// or modifying this code.
//
// Copyright (c) 2013, Gimpy Software.
//
using System;
using System.IO;
using System.Reflection;
using System.Windows.Threading;

namespace Speculator
{
    public class ZxSpectrum : IDisposable
    {
        private const int TStatesPerRenderFrame = 69888;

        public ZxSpectrum(ZXDisplay display)
        {
            TheDisplay = display;
            TheCpu = new CPU(new Memory(64 * 1024), PortHandler, SoundChip)
                {
                    TStatesPerInterrupt = TStatesPerRenderFrame
                };

            TheCpu.RenderCallbackEvent += TheCPU_RenderCallbackEvent;
        }

        private SoundChip m_soundChip;
        private SoundChip SoundChip
        {
            get { return m_soundChip ?? (m_soundChip = new SoundChip()); }
        }

        public CPU TheCpu { get; private set; }
        private ZXDisplay TheDisplay { get; set; }

        private ZXPortHandler m_portHandler;
        public ZXPortHandler PortHandler
        {
            get { return m_portHandler ?? (m_portHandler = new ZXPortHandler(SoundChip)); }
        }

        public void LoadBasicRom()
        {
            string systemRom = Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) ?? ".", "ROMs", "48.rom");
            TheCpu.MainMemory.LoadBasicROM(systemRom);
        }

        private void TheCPU_RenderCallbackEvent(CPU sender, CPU.RenderCallbackArgs args)
        {
            TheDisplay.Bitmap.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => TheDisplay.UpdateScreen(sender)));
        }

        public void Dispose()
        {
            if (m_soundChip != null)
                m_soundChip.Dispose();
        }
    }
}
