﻿// Anyone is free to copy, modify, use, compile, or distribute this software,
// either in source code form or as a compiled binary, for any non-commercial
// purpose.  For commercial enquiries please contact contact@gimpysoftware.com
//
// If modified, please retain this copyright header, and consider telling us
// about your changes.  We're always glad to see how people use our code!
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
// Gimpy Software does not accept any liability for damage caused by executing
// or modifying this code.
//
// Copyright (c) 2013, Gimpy Software.
//
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Input;
using System.Windows.Threading;
using Speculator;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global
namespace SpeculatorApp
{
    // TODO - Add tooltips on registers showing memory content starting at that address.
    class CPUStateViewModel : ViewModelBase
    {
        public CPU TheCPU { get; private set; }

        public CPUStateViewModel(ZxSpectrum theSpectrum)
        {
            TheCPU = theSpectrum.TheCpu;
            RunToAddress = "FFFF";

            UpdateStateFromCPU();
            UpdateStateFromCPU();
        }

        public string A_Color { get; private set; }
        public string F_Color { get; private set; }
        public string B_Color { get; private set; }
        public string C_Color { get; private set; }
        public string D_Color { get; private set; }
        public string E_Color { get; private set; }
        public string H_Color { get; private set; }
        public string L_Color { get; private set; }
        public string Aalt_Color { get; private set; }
        public string Falt_Color { get; private set; }
        public string Balt_Color { get; private set; }
        public string Calt_Color { get; private set; }
        public string Dalt_Color { get; private set; }
        public string Ealt_Color { get; private set; }
        public string Halt_Color { get; private set; }
        public string Lalt_Color { get; private set; }
        public string PC_Color { get; private set; }
        public string SP_Color { get; private set; }
        public string IX_Color { get; private set; }
        public string IY_Color { get; private set; }
        public string I_Color { get; private set; }
        public string R_Color { get; private set; }
        public string IFF1_Color { get; private set; }
        public string IFF2_Color { get; private set; }

        private byte m_A;
        private byte m_F;
        private byte m_B;
        private byte m_C;
        private byte m_D;
        private byte m_E;
        private byte m_H;
        private byte m_L;
        private byte m_Aalt;
        private byte m_Falt;
        private byte m_Balt;
        private byte m_Calt;
        private byte m_Dalt;
        private byte m_Ealt;
        private byte m_Halt;
        private byte m_Lalt;
        private int m_PC;
        private int m_SP;
        private int m_IX;
        private int m_IY;
        private byte m_I;
        private byte m_R;
        private bool m_IFF1;
        private bool m_IFF2;

        public string A { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.A); } }
        public string F { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.F); } }
        public string B { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.B); } }
        public string C { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.C); } }
        public string D { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.D); } }
        public string E { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.E); } }
        public string H { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.H); } }
        public string L { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Main.L); } }
        public string Aalt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.A); } }
        public string Falt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.F); } }
        public string Balt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.B); } }
        public string Calt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.C); } }
        public string Dalt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.D); } }
        public string Ealt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.E); } }
        public string Halt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.H); } }
        public string Lalt { get { return String.Format("{0:X2}", TheCPU.TheRegisters.Alt.L); } }

        public string IX { get { return String.Format("{0:X4}", TheCPU.TheRegisters.IX); } }
        public string IY { get { return String.Format("{0:X4}", TheCPU.TheRegisters.IY); } }
        public string I { get { return String.Format("{0:X2}", TheCPU.TheRegisters.I); } }
        public string R { get { return String.Format("{0:X2}", TheCPU.TheRegisters.R); } }
        public string PC { get { return String.Format("{0:X4}", TheCPU.TheRegisters.PC); } }
        public string SP { get { return String.Format("{0:X4}", TheCPU.TheRegisters.SP); } }

        public string SP_Memory { get { int addr = TheCPU.TheRegisters.SP; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string IX_Memory { get { int addr = TheCPU.TheRegisters.IX; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string IY_Memory { get { int addr = TheCPU.TheRegisters.IY; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string BC_Memory { get { int addr = TheCPU.TheRegisters.Main.BC; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string DE_Memory { get { int addr = TheCPU.TheRegisters.Main.DE; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string HL_Memory { get { int addr = TheCPU.TheRegisters.Main.HL; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string BCalt_Memory { get { int addr = TheCPU.TheRegisters.Alt.BC; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string DEalt_Memory { get { int addr = TheCPU.TheRegisters.Alt.DE; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }
        public string HLalt_Memory { get { int addr = TheCPU.TheRegisters.Alt.HL; return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }

        public string IFF1 { get { return TheCPU.TheRegisters.IFF1 ? "1" : "0"; } }
        public string IFF2 { get { return TheCPU.TheRegisters.IFF2 ? "1" : "0"; } }

        private string m_peekAddr = "0000";
        public string PeekAddr
        {
            get { return m_peekAddr; }
            set
            {
                m_peekAddr = value;
                OnPropertyChanged("PeekAddr");
                OnPropertyChanged("PeekAddr_Memory");
            }
        }
        public string PeekAddr_Memory { get { int addr = Convert.ToInt16(PeekAddr, 16); return String.Format("{0:X4}: {1}", addr, TheCPU.MainMemory.readAsHexString(addr, 8, true)); } }

        public string Flags
        {
            get { return TheCPU.TheRegisters.FlagsAsString(); }
        }

        public string Disassembly
        {
            get
            {
                const int LinesToShow = 8;

                string s = string.Empty;

                int addr = TheCPU.TheRegisters.PC;
                for (int i = 0; i < LinesToShow && addr <= 0xFFFF; i++)
                {
                    if (i > 0)
                        s += "\n";

                    int byteCount;
                    s += GetFormattedDisassembly(addr, out byteCount);

                    addr += byteCount;
                }

                return s;
            }
        }

        private string GetFormattedDisassembly(int addr, out int byteCount)
        {
            string hexBytes = string.Empty;
            string mnemonics = string.Empty;
            byteCount = TheCPU.disassemble(addr, ref hexBytes, out mnemonics);

            return string.Format("{0:X4}", addr) + " : " + hexBytes.PadRight(11) + " : " + mnemonics;
        }

        private readonly List<int> m_oldInstructionAddr = new List<int>();
        public string OldInstructions
        {
            get 
            {
                string s = string.Empty;

                int i = 0;
                foreach (int addr in m_oldInstructionAddr)
                {
                    if (i++ > 0)
                        s += "\n";

                    int byteCount;
                    s += GetFormattedDisassembly(addr, out byteCount);
                }

                return s;
            }
        }

        private RelayCommand m_stepCommand;
        public ICommand OnStep
        {
            get { return m_stepCommand ?? (m_stepCommand = new RelayCommand(x => Step(1))); }
        }

        private RelayCommand m_step10Command;
        public ICommand OnStep10
        {
            get { return m_step10Command ?? (m_step10Command = new RelayCommand(x => Step(10))); }
        }

        private RelayCommand m_RunToCommand;
        public ICommand OnRunTo
        {
            get { return m_RunToCommand ?? (m_RunToCommand = new RelayCommand(x => RunTo(Convert.ToInt32(RunToAddress, 16), true))); }
        }

        private RelayCommand m_ResetCommand;
        public ICommand OnReset
        {
            get { return m_ResetCommand ?? (m_ResetCommand = new RelayCommand(x => Reset())); }
        }

        private RelayCommand m_LoadCommand;
        public ICommand OnLoad
        {
            get { return m_LoadCommand ?? (m_LoadCommand = new RelayCommand(x => Load())); }
        }

        private RelayCommand m_SaveCommand;
        public ICommand OnSave
        {
            get { return m_SaveCommand ?? (m_SaveCommand = new RelayCommand(x => Save())); }
        }

        private RelayCommand m_SaveRunCommand;
        public ICommand OnSaveRun
        {
            get { return m_SaveRunCommand ?? (m_SaveRunCommand = new RelayCommand(x => SaveRun())); }
        }

        private void Load()
        {
            ZXFileFormats.LoadFile(TheCPU);
            IsDebugging = TheCPU.IsDebugging;
            RefreshView();
        }

        private void Save()
        {
            ZXFileFormats.SaveFile(TheCPU);
        }

        private void SaveRun()
        {
            string outPath = Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) ?? ".", "Snapshots");
            if (!Directory.Exists(outPath))
                Directory.CreateDirectory(outPath);

            const int SnapshotsToMake = 1024;
            const int InstructionsToStep = 1;

            for (int i = 0; i < SnapshotsToMake; i++)
            {
                ZXFileFormats.SaveFile(TheCPU, outPath + string.Format(@"\snap_{0}.sna", i));
                Step(InstructionsToStep, false);
            }

            RefreshView();
        }

        private void Reset()
        {
            if (m_runToDispatcherTimer != null)
                m_runToDispatcherTimer.Stop();
            TheCPU.ResetAsync();
            RefreshView();
        }

        private DispatcherTimer m_runToDispatcherTimer;

        private void RunTo(int addr, bool ignoreIfMatch = false)
        {
            if (IsDebugging && (ignoreIfMatch || TheCPU.TheRegisters.PC != addr))
            {
                Step(1, false);

                if (m_runToDispatcherTimer == null)
                {
                    m_runToDispatcherTimer = new DispatcherTimer();
                    m_runToDispatcherTimer.Tick += RunToDispatcherTimerTick;
                }

                m_runToDispatcherTimer.Start();
            }
            else
            {
                RefreshView();
            }
        }

        void RunToDispatcherTimerTick(object sender, EventArgs e)
        {
            m_runToDispatcherTimer.Stop();
            RunTo(Convert.ToInt32(RunToAddress, 16));
        }

        public string RunToAddress { get; set; }

        private void Step(int stepCount, bool refresh = true)
        {
            for (int i = 0; i < stepCount; i++)
            {
                if (refresh)
                {
                    m_oldInstructionAddr.Add(TheCPU.TheRegisters.PC);
                    const int MaxOldInstructions = 4;
                    while (m_oldInstructionAddr.Count > MaxOldInstructions)
                        m_oldInstructionAddr.RemoveAt(0);
                }

                TheCPU.DebuggerTick();

                if (refresh)
                    RefreshView();
            }
        }

        private void RefreshView()
        {
            Thread.Sleep(50);
            UpdateStateFromCPU();
            OnPropertyChanged("");
        }

        private void UpdateStateFromCPU()
        {
            A_Color = (TheCPU.TheRegisters.Main.A == m_A) ? "#000000" : "#FF0000";
            F_Color = (TheCPU.TheRegisters.Main.F == m_F) ? "#000000" : "#FF0000";
            B_Color = (TheCPU.TheRegisters.Main.B == m_B) ? "#000000" : "#FF0000";
            C_Color = (TheCPU.TheRegisters.Main.C == m_C) ? "#000000" : "#FF0000";
            D_Color = (TheCPU.TheRegisters.Main.D == m_D) ? "#000000" : "#FF0000";
            E_Color = (TheCPU.TheRegisters.Main.E == m_E) ? "#000000" : "#FF0000";
            H_Color = (TheCPU.TheRegisters.Main.H == m_H) ? "#000000" : "#FF0000";
            L_Color = (TheCPU.TheRegisters.Main.L == m_L) ? "#000000" : "#FF0000";
            Aalt_Color = (TheCPU.TheRegisters.Alt.A == m_Aalt) ? "#000000" : "#FF0000";
            Falt_Color = (TheCPU.TheRegisters.Alt.F == m_Falt) ? "#000000" : "#FF0000";
            Balt_Color = (TheCPU.TheRegisters.Alt.B == m_Balt) ? "#000000" : "#FF0000";
            Calt_Color = (TheCPU.TheRegisters.Alt.C == m_Calt) ? "#000000" : "#FF0000";
            Dalt_Color = (TheCPU.TheRegisters.Alt.D == m_Dalt) ? "#000000" : "#FF0000";
            Ealt_Color = (TheCPU.TheRegisters.Alt.E == m_Ealt) ? "#000000" : "#FF0000";
            Halt_Color = (TheCPU.TheRegisters.Alt.H == m_Halt) ? "#000000" : "#FF0000";
            Lalt_Color = (TheCPU.TheRegisters.Alt.L == m_Lalt) ? "#000000" : "#FF0000";
            PC_Color = (TheCPU.TheRegisters.PC == m_PC) ? "#000000" : "#FF0000";
            SP_Color = (TheCPU.TheRegisters.SP == m_SP) ? "#000000" : "#FF0000";
            IX_Color = (TheCPU.TheRegisters.IX == m_IX) ? "#000000" : "#FF0000";
            IY_Color = (TheCPU.TheRegisters.IY == m_IY) ? "#000000" : "#FF0000";
            I_Color = (TheCPU.TheRegisters.I == m_I) ? "#000000" : "#FF0000";
            R_Color = (TheCPU.TheRegisters.R == m_R) ? "#000000" : "#FF0000";
            IFF1_Color = (TheCPU.TheRegisters.IFF1 == m_IFF1) ? "#000000" : "#FF0000";
            IFF2_Color = (TheCPU.TheRegisters.IFF2 == m_IFF2) ? "#000000" : "#FF0000";

            m_A = TheCPU.TheRegisters.Main.A;
            m_F = TheCPU.TheRegisters.Main.F;
            m_B = TheCPU.TheRegisters.Main.B;
            m_C = TheCPU.TheRegisters.Main.C;
            m_D = TheCPU.TheRegisters.Main.D;
            m_E = TheCPU.TheRegisters.Main.E;
            m_H = TheCPU.TheRegisters.Main.H;
            m_L = TheCPU.TheRegisters.Main.L;
            m_Aalt = TheCPU.TheRegisters.Alt.A;
            m_Falt = TheCPU.TheRegisters.Alt.F;
            m_Balt = TheCPU.TheRegisters.Alt.B;
            m_Calt = TheCPU.TheRegisters.Alt.C;
            m_Dalt = TheCPU.TheRegisters.Alt.D;
            m_Ealt = TheCPU.TheRegisters.Alt.E;
            m_Halt = TheCPU.TheRegisters.Alt.H;
            m_Lalt = TheCPU.TheRegisters.Alt.L;
            m_PC = TheCPU.TheRegisters.PC;
            m_SP = TheCPU.TheRegisters.SP;
            m_IX = TheCPU.TheRegisters.IX;
            m_IY = TheCPU.TheRegisters.IY;
            m_I = TheCPU.TheRegisters.I;
            m_R = TheCPU.TheRegisters.R;
            m_IFF1 = TheCPU.TheRegisters.IFF1;
            m_IFF2 = TheCPU.TheRegisters.IFF2;
        }

        public bool IsDebugging
        {
            get
            {
                return TheCPU.IsDebugging;
            }
            set
            {
                if (TheCPU.IsDebugging == value) return;
                TheCPU.IsDebugging = value;
                OnPropertyChanged("IsDebugging");

                if (TheCPU.IsDebugging)
                    RefreshView();
            }
        }
    }
}
